program main
  ! Solve 1D time-dependent Schr\"odinger equation
  !                          using sin-basis function.
  ! 
  ! [-\frac{1}{2}(\frac{d^2}{dx^2} + \frac{d^2}{dx^2})
  !                      + V(x,y,t)] \psi(x,y) = E \psi(x,y)
  ! with
  !  \psi(x,y)
  !      = \sum_{k=1} c_k \varphi_{k_x}(x)\varphi_{k_y}(y)
  !   where k_x = k_x(k), k_y = k_y(k),
  !       as ascending order of E=E^{(x)}_{k_x}+E^{(x)}_{k_y}
  !   E^{(x)}_{k_x} = \frac{1}{2}(\frac{k_x \pi}{x_b-x_a})^2
  !   E^{(y)}_{k_y} = \frac{1}{2}(\frac{k_y \pi}{y_b-y_a})^2
  !
  ! Basis : \varphi_k(x)
  !          = \sqrt{2/(x_b-x_a)}\sin(k\pi(x-x_a)/(x_b-x_a))
  !         \varphi_k(y)
  !          = \sqrt{2/(y_b-y_a)}\sin(k\pi(y-y_a)/(y_b-y_a))
  !
  ! Keywords : Quadpack (Gauss-legendre,-Kronrod quadrature)
  !
  ! Author : sikino
  ! Date : 2018/09/12 (yyyy/mm/dd)
  ! URL : http://slpr.sakura.ne.jp/qp/
  ! E-mail : hecate.cb@gmail.com
  ! 
  implicit none
  integer::i,j,k,N,Nx,Ny,Nout
  integer::Ncpu,QPforce
  integer,allocatable::kx(:),ky(:)
  double precision::xa,xb,Ex,x,dx
  double precision::ya,yb,Ey,y,dy
  double precision::QPeps,t0,t1
  double precision,allocatable::Evec(:,:),Eval(:)
  double precision,external::V
  character(48)::fname
  double precision::t,ta
  double precision,parameter::pi=3.14159265358979324d0

  complex(kind(0d0))::fxy
  complex(kind(0d0)),external::wf
  complex(kind(0d0)),allocatable::c(:)

  ! Position x
  xa    = -5d0 ! range [xa,xb]
  xb    =  5d0 !
  Nx    =  50  ! Output number
  ! Position y
  ya    = -5d0 ! range [ya,yb]
  yb    =  5d0 !
  Ny    =  50  ! Output number

  QPeps = 1d-10  ! Integration epsilon, QuadPack

  ! General parameters
  N    = 100    ! Number of sin-basis from 1 to N
  Ncpu = 1      ! Number of CPU threads for openMP
  QPforce = 0   ! Force the integration points
                ! QPforce = 0 : Typical use (default)
                !         = 1 : Force more points 
  Nout = 10     ! Output eigenfunction from n=1 to n=Nout
                ! Nout <= N

  ta=0d0 ! potential time (toward TDSE)
  
  !----------------------
  
  if(Nout.gt.N)Nout=N

  ! For TISE calculation
  !$ call mkl_set_num_threads(Ncpu)
  !$ call omp_set_num_threads(Ncpu)

  allocate(kx(1:N),ky(1:N))
  ! Arrange numerical basis as ascending order of E=Ex+Ey
  call qnumber(N,kx,ky,xa,xb,ya,yb)

  call cpu_time(t0)
  write(6,'(A)')" Solving TISE..."
  allocate(Evec(1:N,1:N),Eval(1:N),c(1:N))
  ! Solve time-independent Schr\"odinger equation
  call tise2d(N,kx,ky,Evec,Eval,V,xa,xb,ya,yb,ta,QPeps,QPforce)
  call cpu_time(t1)
  write(6,'(f10.3,A)')(t1-t0),"[CPU sec]"

  open(10,file="initial_eigenvalue.d")
  do i=1,N
     write(10,*)i,Eval(i)
     if(i.lt.Nout)write(6,*)i,Eval(i)
  enddo  
  close(10)
  
  do k=1,Nout
     write(fname,'(A,i0,A)')"ef_",k,".d"
     open(12,file=trim(fname))

     do i=1,N
        c(i)=dcmplx(Evec(i,k),0d0)
     enddo

     ! Modify the phase, Im(\psi(x,t_a))=0
     do i=1,N
        Ex = 0.5d0*(kx(i)*pi/(xb-xa))**2
        Ey = 0.5d0*(ky(i)*pi/(yb-ya))**2
        c(i)=c(i)*exp(dcmplx(0d0,1d0)*(Ex+Ey)*ta)
     enddo

     t=ta
     dx=(xb-xa)/dble(Nx)
     dy=(yb-ya)/dble(Ny)
     do i=1,Nx
        x=i*dx+xa
        do j=1,Ny
           y=j*dy+ya
           fxy = wf(x,y,t,N,kx,ky,c,xa,xb,ya,yb)
           write(12,'(6e22.10e3)')x,y,dble(fxy),dimag(fxy),V(x,y,t),Eval(k)
        enddo
        write(12,*)
     enddo

     close(12)
  enddo
  
  stop
end program main

function V(x,y,t)
  implicit none
  double precision,intent(in)::x,y,t
  double precision::V
  
  V=0.5d0*x*x+0.5d0*y*y  
  
  return
end function V



!===============================



function wf(x,y,t,N,kx,ky,c,xa,xb,ya,yb)
  implicit none
  integer,intent(in)::N,kx(1:N),ky(1:N)
  double precision,intent(in)::xa,xb,ya,yb,x,y,t
  complex(kind(0d0)),intent(in)::c(1:N)
  complex(kind(0d0))::wf
  
  ! Construct wave fucntion from coefficients
  double precision,parameter::pi=3.14159265358979324d0
  integer::i
  double precision::sx,sy,E

  wf=dcmplx(0d0,0d0)
  do i=1,N
     sx = sqrt(2d0/(xb-xa))*sin(kx(i)*pi*(x-xa)/(xb-xa))
     sy = sqrt(2d0/(yb-ya))*sin(ky(i)*pi*(y-ya)/(yb-ya))
     E  = 0.5d0*(kx(i)*pi/(xb-xa))**2 + 0.5d0*(ky(i)*pi/(yb-ya))**2
     wf = wf + c(i)*sx*sy*exp(-dcmplx(0d0,1d0)*E*t)
  enddo

  return
end function wf


subroutine qnumber(Nmax,kx,ky,xa,xb,ya,yb)
  implicit none
  integer,intent(in)::Nmax
  double precision,intent(in)::xa,xb,ya,yb
  integer,intent(out)::kx(1:Nmax),ky(1:Nmax)

  integer::j,jmax,n,jc
  double precision::Ex,Ey,t,u
  integer,allocatable::s(:)
  double precision,parameter::pi=3.14159265358979324d0
  
  allocate(s(1:Nmax+1))
  kx(1)=1
  ky(1)=1
  s(1)=2
  s(2)=1
  s(3:Nmax+1)=0  
  jmax=2
  
  do n=2,Nmax
     t = 1d100
     jc=1
     do j=1,jmax
        Ex = 0.5d0*(s(j)*pi/(xb-xa))**2
        Ey = 0.5d0*(j*pi/(yb-ya))**2
        u = Ex+Ey 
        if(u.lt.t)then
           jc = j
           t  = u
        endif
     enddo
     kx(n) = s(jc)
     ky(n) = jc
     s(jc) = s(jc)+1
     if(jc.eq.jmax)then
        s(jc+1) = 1
        jmax = jc+1
     endif
  enddo
  
  return
end subroutine qnumber

